#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

char* basename(const char* path, char separator, const char* suffix){
	const char* start = NULL;
	const char* end = NULL;
	char* result = NULL;
	unsigned suffixlen;

	if(path == NULL || suffix == NULL) return NULL;
	if(strchr(suffix, separator) != NULL) return NULL;

	start = strrchr(path, separator);
	if(start == NULL) start = path;
	else start = start + 1;

	end = path + strlen(path);
	suffixlen = strlen(suffix);
	if(strcmp(end - suffixlen, suffix) == 0) end = end - suffixlen;

	result = malloc( (end - start + 1) * sizeof(char) );
	if(result == NULL){
		printf("Greska pri alokaciji memorije.\n");
		exit(EXIT_FAILURE);
	}
	strncpy(result, start, end - start);

	return result;
}


char* detox_and_trim(const char* str, const char* ws, const char* repl){
	const char* start = NULL;
	const char* end = NULL;
	const char* p = NULL;
	char* q = NULL;
	char* result = NULL;

	if(str == NULL || ws == NULL || repl == NULL) return NULL;
	if(strlen(ws) != strlen(repl)) return NULL;
	for(p = ws; *p != '\0'; ++p){
		if(strchr(repl, *p) != NULL) return NULL;
	}

	for(start = str; strchr(ws, *start) != NULL; ++start);
	for(end = str + strlen(str); strchr(ws, *end) != NULL; --end);

    result = malloc( (end - start + 1) * sizeof(char) );
	if(result == NULL){
		printf("Greska pri alokaciji memorije.\n");
		exit(EXIT_FAILURE);
	}
	strncpy(result, start, end - start);
	while(*ws != '\0'){
		for(q = result; *q != '\0'; ++q){
			if(*q == *ws) *q = *repl;
			++repl;
			++ws;
		}
	}

	return result;
}


char* cut(const char* str, int f, const char* delim){
	const char* start = NULL;
	const char* end = NULL;
	char* result = NULL;

	if(str == NULL || delim == NULL) return NULL;
	if(f <= 0) return NULL;

	for(start = str; *start != '\0'; ++start){
		if(strchr(delim, *start) == NULL) --f;
		if(f == 0) break;
	}
	if(f != 0) return NULL;

	for(end = start; end[1] != '\0'; ++end){
		if(strchr(delim, end[1]) == NULL) break;
	}

	result = malloc( (end - start + 1) * sizeof(char) );
	if(result == NULL){
		printf("Greska pri alokaciji memorije.\n");
		exit(EXIT_FAILURE);
	}
	strncpy(result, start, end - start);

	return result;
}


char* nextopt(const char* str){
	const char* start = NULL;
	const char* end = NULL;
	char* result = NULL;

	if(str == NULL) return NULL;

	for(start = strstr(str, "--"); start != NULL && !isalpha(start[2]); start = strstr(start, "--"));
	if(start == NULL) return 0;

	for(end = start; isalpha(*end); ++end);
	if(*end == '='){
		for(end = end + 1; isalpha(*end); ++end);
	}
	--end;

	result = malloc( (end - start + 1) * sizeof(char) );
	if(result == NULL){
		printf("Greska pri alokaciji memorije.\n");
		exit(EXIT_FAILURE);
	}
	strncpy(result, start, end - start);

	return result;
}
